
-- Type of a legacy
Type = { 
	None = 0, 
	Minor = 1, 
	Major = 2, 
	Stat = 3,
	-- default legacies below
	DPS = 4, 
	TacticalDamageRating = 5, 
	TacticalHealingRating = 6, 
	IncomingHealingRating = 7, 
	ShieldUse = 8,
};

-- Modifier type of a legacy, mostly for conveniance in entering data as numeric values rather than strings
Modifier = { 
	None = 0, 
	Number = 1, 
	Percent = 2, 
	Cooldown = 3, 
	Resist = 4, 
	Mitigation = 5, 
	Rating = 6, 
	DPS = 7, 
	Duration = 8, 
	Range = 9
};

local LevelAgeAdjustRanks = 28 -- number of possible modifiers from lvl 51 3rd age to lvl 75 first age for adjustable modifiers

-- how many "free" ranks a legacy get at certain levels, if adjust = LevelRangeAge
LevelRangeBonus = { 
	[51]=0,
	[52]=0,
	[53]=0,
	[54]=1,
	[55]=1,
	[56]=1,
	[57]=1,
	[58]=2,
	[59]=2,
	[60]=2,
	[61]=2,
	[62]=2,
	[63]=2,
	[64]=2,
	[65]=3, 
	[66]=4, 
	[67]=5, 
	[68]=6, 
	[69]=7, 
	[70]=8, 
	[71]=9, 
	[72]=10, 
	[73]=11, 
	[74]=12, 
	[75]=13,
};

-- Adjust functions 

-- returns the number of "free" ranks a legacy gets at a given level & age
local LevelRangeAgeBonus = function( level, age )
	local bonus = LevelRangeBonus[level];
	bonus = bonus + 3 - age;
	if level == 75 and ( age == 2 or age == 1 ) then
		bonus = bonus + 4;
	end
	return bonus;
end

-- return a cost times a multiplier using table values if necessary (if not, may be off due to rounding error)
local CostMult = function( multiplier, multTable, costTable, rank )
	if multTable ~= nil then
		local cost = multTable[rank-1];
		if cost ~= 0 then
			return cost;
		end
	end
	return math.floor( costTable[rank-1] * multiplier );
end

local HalfCost = function( costTable, rank ) return CostMult( 0.5, costTable.Half, costTable, rank ); end	
local OneAndHalfCost = function( costTable, rank ) return CostMult( 1.5, costTable.OneAndHalf, costTable, rank ); end
local DoubleCost = function( costTable, rank ) return CostMult( 2, costTable.Double, costTable, rank ); end
local TripleCost = function( costTable, rank ) return CostMult( 3, costTable.Triple, costTable, rank ); end


-- ADJUSTMENT FUNCTIONS FOR COSTS & VALUES & # of RANKS

-- Group: Tactical Healing Rating

-- Modifiers are shifted by 1 for each level, +1 for each age < 3rd, +4 for level 75 2nd and 1st age
local ModAdjustForLevelAndAge = function( modifiers )
	return function ( level, age, rank )
		local bonus = level - 51 + (3 - age);
		if level == 75 and ( age == 2 or age == 1 ) then
			bonus = bonus + 4;
		end
		return modifiers[bonus + rank];
	end
end

-- Group: DPS rating, Tactical Damage Rating, Shield Use

-- Modifiers are shifted by 1 for each level, +4 for lvl 75 2nd and 1st ages. Different table / age
local ModAdjustForLevelPerAge = function( modifiers )
	return function( level, age, rank )
		local bonus = level - 51;
		if level == 75 and ( age == 2 or age == 1 ) then
			bonus = bonus + 4;
		end
		return modifiers[age][bonus + rank];
	end
end

-- Group: Incoming Healing Rating

-- Different tables for level ranges 51..60, 61..65, 66..75, different rables / age
local ModAdjustForIncomingHealing = function( modifiers )
	return function( level, age, rank )
		-- different tables for 1st, 2nd, third age & for lvl 51-60 & 61-65 ranges
		if level <= 60 then
			return modifiers[1][age][rank]; -- lvl 51-60 table
		elseif level <= 65 then
			return modifiers[2][age][rank]; -- lvl 61+ table
		else 
			return modifiers[3][age][rank]; -- lvl 66+ table
		end
	end
end

-- Group: most rating & threat rating legacies, as well as certain +pulses & +range

-- Modifiers shifted according to the default shifting mechanism
local ModAdjustForLevelRangeAndAge = function( modifiers )
	return function( level, age, rank )
		local bonus = LevelRangeAgeBonus( level, age );
		return modifiers[bonus + rank];
	end
end

-- Group: Pulse1

-- One fewer rank at lvl 75 2nd age (& first, though that's just a guess) TODO: Check for first ages (7 ranks ?)
local MaxRankAdjustPulse = function()
	return function( level, age )
		if level == 75 and ( age == 2 or age == 1 ) then
			return 8;
		end
		return 9;
	end
end

-- Gain half R2 cost every other bonus
local ModAdjustPulse = function()
	return function( level, age, rank )
		local bonus = LevelRangeAgeBonus( level, age );
		return rank + math.floor( bonus / 2 );
	end
end

-- Gain +1 pulse every other bonus 
local CostAdjustPulse = function( costs )
	return function( level, age, tier, rank )
		if rank == 2 then 
			local bonus = LevelRangeAgeBonus( level, age );
			if math.mod( bonus, 2 ) == 1 then
				return HalfCost( costs[tier], rank );
			end
		end
		return costs[tier][rank-1];
	end
end

-- Group: Range

-- Gains +1 range at bonus 1, 2 then every other bonus
local ModAdjustRange = function( modifiers )
	return function( level, age, rank )
		local bonus = LevelRangeAgeBonus( level, age );
		if bonus == 0 then
			return modifiers[rank];
		end
		return modifiers[rank + math.floor( bonus / 2 ) + 1];
	end
end

-- half R2 & R3 costs @ bonus 0, half R2 costs every other bonus, 2x shifting costs at higher bonus levels
local CostAdjustRange = function( costs )
	return function( level, age, tier, rank )
		local bonus = LevelRangeAgeBonus( level, age );
		if rank == 2 then 
			if math.mod( bonus, 2 ) == 1 or bonus == 0 then
				return HalfCost( costs[tier], rank );
			end
		elseif bonus == 0 and rank == 3 then
			return HalfCost( costs[tier], 3 );
		elseif bonus > 6 then
			local multiplierRank = 9 - math.floor( ( bonus - 6 ) / 2 );
			if rank == multiplierRank then
				return OneAndHalfCost( costs[tier], rank );
			end
		end
		return costs[tier][rank-1];
	end
end

-- Group: SoS

-- +1 value shift / bonus, apparently capped
local ModAdjustSoS = function( modifiers )
	return function( level, age, rank )
		local bonus = LevelRangeAgeBonus( level, age );
		bonus = math.min( 17, bonus );
		return modifiers[rank + bonus];
	end
end

-- shifting *2 & *3 cost multipliers at higher bonus levels
local CostAdjustSoS = function( costs )
	return function( level, age, tier, rank )
		local bonus = LevelRangeAgeBonus( level, age );
		bonus = math.min( 17, bonus );
		local multiplierRank = 9 - bonus + 9;
		if rank == multiplierRank then
			return DoubleCost( costs[tier], rank );
		elseif rank == multiplierRank + 1 then
			return TripleCost( costs[tier], rank );
		end
		
		return costs[tier][rank - 1];
	end
end

-- Group: Threat

-- 'skips' a value at a threshold ...
local ModAdjustThreat = function( modifiers, threshold )
	return function( level, age, rank )
		local bonus = LevelRangeAgeBonus( level, age );
		if threshold > bonus + 1 then
			if bonus + rank >= threshold then
				return modifiers[rank + bonus + 1];
			end
		end
		return modifiers[rank + bonus];
	end
end

-- double shifting costs 
local CostAdjustThreat = function( costs, threshold )
	return function( level, age, tier, rank )
		local bonus = LevelRangeAgeBonus( level, age );
		if bonus + rank == threshold then
			return DoubleCost( costs[tier], rank );
		end
		return costs[tier][rank-1];
	end
end

-- Groups: Target, Pulses 2, PiP Interval

-- max rank adjustment in a modulo
local MaxRankAdjustModulo = function( base, modulo )
	return function( level, age )
		local bonus = LevelRangeAgeBonus( level, age );
		return base - math.min( 2, math.floor( bonus / modulo ) );
	end
end

-- Modifiers shifted according to the default shifting mechanism at 1/x the rate
local ModAdjustModulo = function( modifiers, modulo )
	return function( level, age, rank )
		local bonus = LevelRangeAgeBonus( level, age );
		local shift = math.min( 2, math.floor( bonus / modulo ) ); -- hack: tables get extended after going round twice
		return modifiers[rank + shift];
	end
end

-- Variable R2 costs 
local CostAdjustModulo = function( costTable, adjustTable, modulo, shift )
	return function( level, age, tier, rank )
		local bonus = LevelRangeAgeBonus( level, age );
		if rank == 2 then 
			-- Turbine.Log:Information( tostring( bonus/modulo ) );
			if bonus / modulo >= 2 then
				return adjustTable[tier][(bonus-(2*modulo) ) + 1 + math.max( 0, shift - 2) ];
			else
				return adjustTable[tier][math.mod( bonus,modulo ) + 1 + shift ];
			end
		end
		local tableTier = math.min( 3, math.floor( bonus/modulo ) + 1 );
		return costTable[tableTier][tier][rank-1];
	end
end

local CostAdjustModuloTarget = function( costTable, adjustTable )
	return CostAdjustModulo( costTable, adjustTable, 7, 3 );
end

local CostAdjustModuloPulse = function( costTable, adjustTable )
	return CostAdjustModulo( costTable, adjustTable, 6, 4 );
end

local CostAdjustModuloPipInterval = function( costTable, adjustTable )
	return CostAdjustModulo( costTable, adjustTable, 8, 2 );
end

-- Group: Stealth
-- Essentially the same as modulo above, but going round cost tables a bit more erratically

local ModAdjustStealth = function( modifiers )
	return function( level, age, rank )
		local bonus = LevelRangeAgeBonus( level, age );
		local shift;
		if bonus < 9 then
			shift = 0;
		elseif bonus < 15 then
			shift = 1;
		else
			shift = 2;
		end
		return modifiers[rank+shift];
	end
end

local MaxRankStealth = function(level,age )
	local bonus = LevelRangeAgeBonus( level, age );
	if bonus < 9 then
		return 3;
	elseif bonus < 15 then
		return 2;
	else
		return 1;
	end
end

local MaxRankAdjustStealth = function()
	return MaxRankStealth;
end

local CostAdjustStealth = function( costTable, adjustTable )
	return function( level, age, tier, rank )
		local bonus = LevelRangeAgeBonus( level, age );
		if rank == 2 then 
			if bonus < 9 then
				return adjustTable[tier][bonus + 1];
			else
				return adjustTable[tier][bonus - 9 + 3 + 1 ];
			end
		end
		local tableTier = 3 - MaxRankStealth( level, age ) + 1;
		-- math.min( 3,  math.floor( bonus/4 ) + 1 );
		return costTable[tableTier][tier][rank-1];
	end
end

-- Group: Harmstring Range
-- Essentially the same as modulo above, but going round cost tables a bit more erratically
local ModAdjustHarmstringRange = function( modifiers )
	return function( level, age, rank )
		local bonus = LevelRangeAgeBonus( level, age );
		local shift;
		if bonus < 4 then
			shift = 0;
		elseif bonus < 10 then
			shift = 1;
		else
			shift = 2;
		end
		return modifiers[rank+shift];
	end
end

local MaxRankHarmstringRange = function( level, age )
	local bonus = LevelRangeAgeBonus( level, age );
	if bonus < 4 then
		return 3;
	elseif bonus < 10 then
		return 2;
	else
		return 1;
	end
end

local MaxRankAdjustHarmstringRange = function()
	return MaxRankHarmstringRange;
end

local CostAdjustHarmstringRange = function( costTable, adjustTable )
	return function( level, age, tier, rank )
		local bonus = LevelRangeAgeBonus( level, age );
		if rank == 2 then 
			if bonus > 3 then
				return adjustTable[tier][bonus - 4 + 1 + 4 ];
			else
				return adjustTable[tier][bonus + 1 + 6 ];
			end
		end
		local tableTier = 3 - MaxRankHarmstringRange( level, age ) +1;
		-- math.min( 3,  math.floor( bonus/4 ) + 1 );
		return costTable[tableTier][tier][rank-1];
	end
end

-- Group: Targeted Melee Skill Range
-- Same as rating, but modifiers capped, resulting in the legacy losing ranks at higher levels

local ModAdjustForLevelRangeAndAgeCap = function( modifiers, cap )
	return function( level, age, rank )
		local bonus = LevelRangeAgeBonus( level, age );
		return modifiers[math.min( 9 + cap, bonus + rank)];
	end
end

local MaxRankAdjustCap = function( cap )
	return function( level, age )
		local bonus = LevelRangeAgeBonus( level, age );
		return math.max( 1, 9 - math.max( 0, bonus - cap ) );
	end
end

------------------------------------ LEGACY DATA ---------------------------

-- Costs of a default legacy per rank. Default legacies don't have tiers, so only one inner table.
CostTableDefault = { {14, 21, 30, 41, 54, 70 } };

-- REGULAR LEGACIES

-- not used in regular tables, but useful for *2 or *3  multipliers. 0 values are placeholders, should be replaced with multiplication for default
Cost378 = {0,0,0,0,0,0,0,0};
Cost324 = {0,0,0,0,0,210,264,324};
Cost270 = {0,0,0,0,135,175,0,0}; -- probably 269
Cost162 = {0,0,0,0,0,105,132,0};
Cost135 = {0,0,0,0,68,88,0,0};
Cost117 = {0,0,0,44,59,0,0,117};
Cost99 = {0,0,0,38,50,64,81,99};
Cost82 = {0,0,0,31,41,0,0,0}; -- probably 81

-- Tables of  legacy costs / rank that most legacies use at various tiers. Tables are numbered by their rank 9 cost for 'easy' identification
Cost252 = 	{32,48,69,95,126,163,205,252 };
Cost216 =	{27,41,59,81,108,140,176,216 };
Cost180 =	{23,34,49,68,90,117,147,180};
Cost144 =	{18,27,39,54,72,93,117,144};

Cost126 = 	{16,24,35,48,63,82,103,126 };
Cost108 = 	{14,21,30,41,54,70,88,108 };
Cost90 =	{12,17,25,34,45,59,74,90 };
Cost72 = 	{9,14,20,27,36,47,59,72 };

Cost63 =	{8,12,18,24,32,41,52,63};
Cost54 =	{7,11,15,21,27,35,44,54 };
Cost45 =	{6,9,13,17,23,30,37,45 };
Cost36 =	{5,7,10,14,18,24,30,36 };

Cost32 =	{4,6,9,12,16,21,26,32};
Cost27 =	{4,6,8,11,14,18,22,27};
Cost23 =	{3,5,7,9,12,15,19,23};
Cost18 =	{3,4,5,7,9,12,15,18 };

Cost59 =	{8,11,16,22,30,38,48,59}; -- TODO 38 was 36 previously, was probably erroneous, need to keep an eye on it
Cost50 = 	{7,10,14,19,25,32,41,50};
Cost41 = 	{6,8,11,16,21,27,33,41};

Cost252.Half = Cost126;
Cost216.Half = Cost108;
Cost180.Half = Cost90;
Cost144.Half = Cost72;
Cost126.Half = Cost63;
Cost108.Half = Cost54;
Cost90.Half = Cost45;
Cost72.Half = Cost36;
Cost63.Half = Cost32;
Cost54.Half = Cost27;
Cost45.Half = Cost23;
Cost36.Half = Cost18;

Cost252.OneAndHalf = Cost378;
Cost216.OneAndHalf = Cost324;
Cost180.OneAndHalf = Cost270;
Cost144.OneAndHalf = Cost216;
-- Cost126.OneAndHalf = Cost189 needed, no data
Cost108.OneAndHalf = Cost162;
Cost90.OneAndHalf = Cost135 
Cost72.OneAndHalf = Cost108;
-- Cost63.OneAndHalf = Cost95 needed, no data
Cost54.OneAndHalf = Cost82;
Cost36.OneAndHalf = Cost54;

Cost126.Double = Cost252;
Cost108.Double = Cost216;
Cost90.Double = Cost180;
Cost72.Double = Cost144;
Cost63.Double = Cost126;
Cost54.Double = Cost108;
Cost45.Double = Cost90;
Cost36.Double = Cost72;
Cost32.Double = Cost63;
Cost27.Double = Cost54;
Cost23.Double = Cost45;
Cost18.Double = Cost36;
Cost59.Double = Cost117;
Cost50.Double = Cost99;
Cost41.Double = Cost82;

Cost126.Triple = Cost378;
Cost108.Triple = Cost324;
Cost90.Triple = Cost270;
Cost72.Triple = Cost216;
Cost54.Triple = Cost162;
Cost45.Triple = Cost135;
Cost36.Triple = Cost108;
-- Cost32.Triple = Cost96, needed, no data
Cost27.Triple = Cost82;
-- Cost23.Triple = needed, no data
Cost18.Triple = Cost54;

-- Tables of legacy costs per rank per tier, that are used for most legacies. Tables are numbered by their tier6/rank9 cost for 'easy" identification
CostTable72 = { Cost252, Cost216, Cost180, Cost144, Cost108, Cost72 }; -- 36*(8-t)  ?
CostTable63 = { Cost216, Cost180, Cost144, Cost108, Cost72, Cost63 };
CostTable54 = { Cost180, Cost144, Cost108, Cost72, Cost63, Cost54 }; -- 214

CostTable36 = { Cost126, Cost108, Cost90, Cost72, Cost54, Cost36 }; -- total at rank 9 = 144 points tor tier 6
CostTable32 = { Cost108, Cost90, Cost72, Cost54, Cost36, Cost32 }; -- total at rank 9 = 126 points tor tier 6 

CostTable27 = { Cost90, Cost72, Cost54, Cost36, Cost32, Cost27 }; -- total at rank 9 = 110 points for tier 6

CostTable27b = { Cost108, Cost72, Cost50, Cost41, Cost32, Cost27 };

CostTable23 = { Cost72, Cost59, Cost50, Cost41, Cost32, Cost23 }; --total at rank 9 = 93 points for tier 6

CostTable18 = { Cost63, Cost54, Cost45, Cost36, Cost27, Cost18 }; -- total at rank 9 = 73 points for tier 6

CostTable18b = { Cost54, Cost45, Cost36, Cost27, Cost23, Cost18 }; -- total at rank 9 = 73 points for tier 6

CostAdjust1 = {158,142,126,111,95,79,63,48,32,16};
CostAdjust2 = {135,122,108,95,81,68,54,41,27,14};
CostAdjust3 = {113,102,90,79,68,57,45,34,23,12};
CostAdjust4 = {90,81,72,63,54,45,36,27,18,9};
CostAdjust5 = {80,72,64,56,48,40,32,24,16,8};
CostAdjust6 = {68,61,54,48,41,34,27,21,14,7};
CostAdjust7 = {56,50,45,40,34,28,23,17,12,6};
CostAdjust8 = {45,41,36,32,27,23,18,14,9,5};
CostAdjust9 = {40,36,32,28,24,20,16,12,8,4};
CostAdjust10 = {34,31,27,24,21,17,14,11,7,3};
CostAdjust11 = {0,0,18,16,12,11,9,7,6,2};

-- Call of the Second Age Targets (MNS), Ancient Craft Targets (LRM), Bane Flare Target (LRM), Fire-lore Targets (LRM), Guardian Area Effect Melee Targets (GRD)
CostTableAdjustTarget1 = { CostAdjust1, CostAdjust2, CostAdjust3, CostAdjust4, CostAdjust6, CostAdjust8 };
-- Pressing Attack Max Targets (CPT)
CostTableAdjustTarget2 = { CostAdjust2, CostAdjust3, CostAdjust4, CostAdjust6, CostAdjust8, CostAdjust9 };
-- Challenge Targets (GRD), Vivid Imagery Targets (RNK), Maximum Targets for AoE Skills (HNT)
CostTableAdjustTarget3 = { CostAdjust3, CostAdjust4, CostAdjust6, CostAdjust8, CostAdjust9, CostAdjust10 };

-- Rend pulses (CHP), Guardian Bleed Pulses (GRD)
CostTableAdjustPulse1 = { CostAdjust1, CostAdjust2, CostAdjust3, CostAdjust4, CostAdjust6, CostAdjust8 };
-- Pulse Modifier for Glee (BRG)
CostTableAdjustPulse2 = { CostAdjust4, CostAdjust6, CostAdjust7, CostAdjust8, CostAdjust10, CostAdjust11 };
-- Wizard's Fire Pulses (LRM), Burning Embers Pulses (LRM)
CostTableAdjustPulse3 = { CostAdjust5, CostAdjust6, CostAdjust7, CostAdjust8, CostAdjust10, CostAdjust11 };
-- Stealth Detection (HNT), Stealth Level (BRG), Harmstring Range(CHP), Ardour/Glory Pip Interval (CHP)

CostTableAdjustStealth = CostTableAdjustTarget1;
CostTableAdjustPipInterval = CostTableAdjustTarget1;
CostTableAdjustHarmstringRange = CostTableAdjustTarget1;

-- TARGET LEGACIES

-- Tables of legacy costs per rank for +target legacies @ various tiers
-- Five ranks, rank 2 variable
CostTableTarget1_1 = {0,166,308,378};
CostTableTarget1_2 = {0,142,264,324};
CostTableTarget1_3 = {0,119,220,270};
CostTableTarget1_4 = {0,95,176,216};
CostTableTarget1_5 = {0,71,132,162};
CostTableTarget1_6 = {0,48,88,108};
CostTableTarget1_7 = {0,42,77,95};
CostTableTarget1_8 = {0,36,66,81};

-- Four ranks, rank 2 variable
CostTableTarget2_1 = {0,213,273};
CostTableTarget2_2 = {0,183,234};
CostTableTarget2_3 = {0,152,195};
CostTableTarget2_4 = {0,122,156};
CostTableTarget2_5 = {0,92,117};
CostTableTarget2_6 = {0,61,78};
CostTableTarget2_7 = {0,54,69};
CostTableTarget2_8 = {0,46,58};

-- Three ranks, rank 2 variable
CostTableTarget3_1 = {0,189};
CostTableTarget3_2 = {0,162};
CostTableTarget3_3 = {0,135};
CostTableTarget3_4 = {0,108};
CostTableTarget3_5 = {0,81};
CostTableTarget3_6 = {0,54};
CostTableTarget3_7 = {0,47};
CostTableTarget3_8 = {0,41};

CostTableTarget1 = { -- Call of the Second Age Targets (MNS), Ancient Craft Targets (LRM), Bane Flare Target (LRM), Fire-lore Targets (LRM), Guardian Area Effect Melee Targets (GRD)
	{ CostTableTarget1_1, CostTableTarget1_2, CostTableTarget1_3, CostTableTarget1_4, CostTableTarget1_5, CostTableTarget1_6 },
	{ CostTableTarget2_1, CostTableTarget2_2, CostTableTarget2_3, CostTableTarget2_4, CostTableTarget2_5, CostTableTarget2_6 },
	{ CostTableTarget3_1, CostTableTarget3_2, CostTableTarget3_3, CostTableTarget3_4, CostTableTarget3_5, CostTableTarget3_6 }
};

CostTableTarget2 = { -- Pressing Attack Max Targets (CPT)
	{ CostTableTarget1_2, CostTableTarget1_3, CostTableTarget1_4, CostTableTarget1_5, CostTableTarget1_6, CostTableTarget1_7 },
	{ CostTableTarget2_2, CostTableTarget2_3, CostTableTarget2_4, CostTableTarget2_5, CostTableTarget2_6, CostTableTarget2_7 },
	{ CostTableTarget3_2, CostTableTarget3_3, CostTableTarget3_4, CostTableTarget3_5, CostTableTarget3_6, CostTableTarget3_7 }
};

CostTableTarget3 = {  -- Challenge Targets (GRD), Vivid Imagery Targets (RNK), Maximum Targets for AoE Skills (HNT)
	{ CostTableTarget1_3, CostTableTarget1_4, CostTableTarget1_5, CostTableTarget1_6, CostTableTarget1_7, CostTableTarget1_8 },
	{ CostTableTarget2_3, CostTableTarget2_4, CostTableTarget2_5, CostTableTarget2_6, CostTableTarget2_7, CostTableTarget2_8 },
	{ CostTableTarget3_3, CostTableTarget3_4, CostTableTarget3_5, CostTableTarget3_6, CostTableTarget3_7, CostTableTarget3_8 }
};

-- PULSE LEGACIES

CostTablePulse1_1 = {0,142,273,331,376};
CostTablePulse1_2 = {0,122,234,284,324};
CostTablePulse1_3 = {0,102,195,237,270};
CostTablePulse1_4 = {0,81,156,189,215};
CostTablePulse1_4b = {0,71,137,166,189};
CostTablePulse1_5 = {0,61,117,142,162};
CostTablePulse1_6 = {0,51,98,119,135};
CostTablePulse1_7 = {9,41,78,95,108};
CostTablePulse1_8 = {0,31,59,71,81};
CostTablePulse1_9 = {0,21,39,48,54};

CostTablePulse2_1 = {0,189,240,294};
CostTablePulse2_2 = {0,162,206,244};
CostTablePulse2_3 = {0,135,172,204};
CostTablePulse2_4 = {0,108,137,162};
CostTablePulse2_4b = {0,95,120,142};
CostTablePulse2_5 = {0,81,103,122};
CostTablePulse2_6 = {0,68,86,102};
CostTablePulse2_7 = {0,54,69,81};
CostTablePulse2_8 = {0,41,52,61};
CostTablePulse2_9 = {0,27,34,41};

CostTablePulse3_1 = {0,166,205};
CostTablePulse3_2 = {0,142,176};
CostTablePulse3_3 = {0,118,147};
CostTablePulse3_4 = {0,95,117};
CostTablePulse3_4b = {0,83,103};
CostTablePulse3_5 = {0,71,88};
CostTablePulse3_6 = {0,60,74};
CostTablePulse3_7 = {0,48,59};
CostTablePulse3_8 = {0,36,44};
CostTablePulse3_9 = {0,24,29};

CostTablePulse1 = { -- Rend Pulses(CHP), Guardian bleed pulses (GRD)
	{ CostTablePulse1_1, CostTablePulse1_2, CostTablePulse1_3, CostTablePulse1_4, CostTablePulse1_5, CostTablePulse1_7 },
	{ CostTablePulse2_1, CostTablePulse2_2, CostTablePulse2_3, CostTablePulse2_4, CostTablePulse2_5, CostTablePulse2_7 },
	{ CostTablePulse3_1, CostTablePulse3_2, CostTablePulse3_3, CostTablePulse3_4, CostTablePulse3_5, CostTablePulse3_7 }
};

CostTablePulse2 = { -- Pulse Modifier for Glee (BRG)
	{ CostTablePulse1_4, CostTablePulse1_5, CostTablePulse1_6, CostTablePulse1_7, CostTablePulse1_8, CostTablePulse1_9 },
	{ CostTablePulse2_4, CostTablePulse2_5, CostTablePulse2_6, CostTablePulse2_7, CostTablePulse2_8, CostTablePulse2_9 },
	{ CostTablePulse3_4, CostTablePulse3_5, CostTablePulse3_6, CostTablePulse3_7, CostTablePulse3_8, CostTablePulse3_9 }
};

CostTablePulse3 = { -- Burning Ember Pulses (LRM) - Wizard Fire Pulses (LRM)
	{ CostTablePulse1_4b, CostTablePulse1_5, CostTablePulse1_6, CostTablePulse1_7, CostTablePulse1_8, CostTablePulse1_9 },
	{ CostTablePulse2_4b, CostTablePulse2_5, CostTablePulse2_6, CostTablePulse2_7, CostTablePulse2_8, CostTablePulse2_9 },
	{ CostTablePulse3_4b, CostTablePulse3_5, CostTablePulse3_6, CostTablePulse3_7, CostTablePulse3_8, CostTablePulse3_9 }
};

-- STEALTH LEVEL

-- Stealth Level (BRG), Stealth Detection (HNT)
CostTableStealth = {
	{ {0,142},{0,122},{0,102},{0,81},{0,61},{0,41} },
	{ {0},{0},{0},{0},{0},{0} },
	{ {},{},{},{},{},{} }
};
-- MISC

-- Ardour/Glory Pip Interval (CHP)
CostTablePipInterval = {
	{ {0,189,376}, {0,162,322}, {0,135,269}, {0,108,215}, {0,81,161}, {0,54,108} },
	{ {0,260}, {0,223}, {0,186}, {0,149}, {0,112}, {0,75} },
	{ {0},{0},{0},{0},{0},{0} }
}

-- Harmstring Range
CostTableHarmstringRange = CostTableStealth;
	

-- Creates a table of values for Adjust.LevelRangeAge, starting from <base>, each succsessive value is increased by <increment>
ModifierTableRating = function( base, increment ) 
	local tab = {};
	for i = 1, LevelAgeAdjustRanks do -- TODO: RoI find correct value
		tab[i] = base + ( ( i - 1 ) * increment );
	end
	
	return tab;
end

NegTable = function( tab )
	local newtab = {};
	for i = 1, #tab do
		newtab[i] = -tab[i];
	end
	return newtab;
end

-- Various rating tables in use by a number of legacies
-- values from lvl 51 3rd age rank 1 to lvl 75 2nd age rank 9
ModifierTableRating207_19 = {207,226,245,264,283,302,321,340,359,378,397,416,435,454,473,492,511,530,536,544,552,560,568,576,584,592,600,608};
ModifierTableRating460_50 = {460,510,560,610,660,710,760,810,860,910,960,1010,1060,1110,1160,1210,1260,1280,1300,1320,1340,1360,1380,1400,1420,1440,1460,1480}; 
ModifierTableRating1105_105 = {1105,1210,1315,1420,1525,1630,1735,1840,1945,2050,2155,2260,2365,2470,2575,2680,2785,2890,2995,3036,3080,
	3124,3168,3212,3256,3300,3344,3388}; -- extrapolated
ModifierTableThreat = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29};

ModifierTableRating207_19_neg = NegTable( ModifierTableRating207_19 );
ModifierTableRating460_50_neg = NegTable( ModifierTableRating460_50 );
ModifierTableRating1105_105_neg = NegTable( ModifierTableRating1105_105 );
ModifierTableThreat_neg = NegTable( ModifierTableThreat );
	
-- MODIFIER VALUES OF DEFAULT LEGACIES
--NOTE: Tactical classes have their melee dps set as if rank 4

ModifierTableTacticalHealingRating = {0.5,0.8,0.9,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35 }; 


-- TODO RoI
-- lvl 65 2nd age: 13, 14,15,15.7,17,18.3,19.9
-- lvl 65 3rd age 12, 13,14,15.1,16.3,17.6,19.1
-- lvl 68 3rd age
ModifierTableTacticalDamageRating = {
	{0.9,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20, -- first age -> lvl 65 rank 7
	-1,-1,-1,-1,-1,-1,-1,41.9,44.7,47.6,50.4,53.3,56.1,59.1}, -- 1st age lvl 66+, TODO RoI, extrapolated
	{0.8,0.9,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,15.7,17,18.3,19.9, -- 2nd age -> lvl 65 rank 7. Values before 13 pre RoI, TODO check
	-1,-1,-1,-1,-1,-1,-1,39.7,42.4,45.1,47.8,50.5,53.2,56.0}, -- 2nd age lvl 66+, lvl 75 2nd age assumed to be shifted +4
	{0.5,0.8,0.9,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15.1,16.3,17.6,19.1, --third age -> lvl 65 rank 7
	20.9,23,25.3,27.8,30.4,33,35.6,38.2,40.8,43.4}, -- 3rd age lvl 66+
};

ModifierTableShieldUseRating = ModifierTableTacticalDamageRating;

-- Post RoI. lvl 65 rank 7 seems to have changed accross the board
ModifierTableDPS1H = { -- DPS tables for one handed weapons
	{-1,-1,-1,-1,-1,-1,-1,-1,50.8,52.8,54.8,56.9,58.4,60.0,61.5,63.0,64.5,67.8,71.1,74.7,78.5, -- first age lvl 51 rank 1 -> level 65 rank 7
	 -1,-1,-1,-1,-1,-1,-1,107,110.6,114.2,117.8,121.2,124.8,128.4}, -- TODO RoI, extrapolated
	{-1,-1,38.5,39.4,40.6,41.7,43.8,46.0,48.3,50.1,52.1,54.1,55.5,57.0,58.4,59.9,61.3,64.4,67.6,71.0,74.4, -- second age, lvl 51 rank 1 -> lvl 65 rank 7
	 -1,-1,-1,-1,-1,-1,-1,101.4,104.8,108.2,111.6,114.9,118.3,121.7}, -- -> lvl 75 rank 1..7, 
	 -- extrapolated from 3rd age: 77.9,81.2,84.6,87.9,91.4,94.7,98.1,101.5,104.9,108.2 would match if lvl +4
	{35.7,36.4,37.1,38.0,39.0,40.1,42.1,44.2,46.5,48.3,50.1,52.0,53.4,54.8,56.2,57.6,59.0,62.0,65.1,68.3,71.6, -- third age, lvl 51 rank 1 -> lvl 65 rank 7
	 74.9,78.1,81.4,84.6,87.9,91.1,94.4,97.6,100.9,104.1}, -- -> lvl 75 rank 7 (RoI)
};
ModifierTableDPS2H = { -- DPS tables for bows, crossbows, javelins & any two handed weapon
	{-1,-1,-1,-1,-1,-1,-1,-1,57.6,59.8,62.1,64.5,66.2,68.0,69.7,71.4,73.1,76.8,80.7,84.7,88.9, -- first age lvl 51 rank 1 -> level 65 rank 7
	 -1,-1,-1,-1,-1,-1,-1,121.3,125.3,129.3,133.4,137.4,141.4,145.4}, -- TODO RoI, extrapolated
	{-1,-1,42.4,44.7,46.0,47.3,49.6,52.1,54.7,56.8,59.0,61.3,62.9,64.6,66.2,67.9,69.5,73.0,76.7,80.5,84.3, -- second age, lvl 51 rank 1 -> lvl 65 rank 7.(last value changed in RoI)  (42.4 extrapolated)
	 -1,-1,-1,-1,-1,-1,-1,115.0,118.8,122.6,126.5,130.3,134.1,137.9}, -- -> lvl 75 rank 7
	 -- extrapolated from 3rd age: 88.1,92,96.6,99.7,103.5,107.4,111.2,115,118.8,122.7 would match if lvl +4
	{40.7,41.4,42.2,43.0,44.2,45.5,47.8,50.1,52.7,54.7,56.8,59.0,60.6,62.1,63.7,65.3,66.9,70.3,73.8,77.5,81.1, -- third age lvl 51 rank 1 -> lvl 65 rank 7
	 84.8,88.5,92.9,95.9,99.6,103.3,107,110.6,114.3,118}, -- -> lvl  75 rank 7 (RoI)
};

ModifierTableIncomingHealingRating = { --incoming healing rating modifiers
	{ -- lvl 51-60: 1st, 2nd, 3rd age
		{720,920,1120,1320,1520,1720,1920}, -- lvl 59 1st age
		{480,680,880,1080,1280,1480,1680}, -- lvl 51-60 2nd age -- TODO CHECK (checked 58)
		{240,440,640,840,1040,1240,1440}, -- lvl 51 - 60 third age TODO CHECK (checked 51,54,56,57,59,60)
	}, { -- lvl 61-65: 1st, 2nd, 3rd age
		{780,996,1213,1430,1646,1863,2080}, -- extrapolated values, TODO check
		{520,736,953,1170,1386,1603,1820}, -- lvl 65 2nd age
		{260,476,693,910,1126,1343,1560}, -- third age checked 61 - 65	
		--TODO RoI
	}, { -- lvl 66+ 1st, 2nd, 3rd age
		{780,996,1213,1430,1646,1863,2080}, -- lvl 66+ guess same as lvl 61-65 TODO check
		{520,736,953,1170,1386,1603,1820}, -- lvl 66+  guess same as lvl 61-65
		{240,440,640,840,1040,1240,1440}, -- lvl 66 checked, same as lvl 51-60 (!)
	}
}


-- COMPLETE LISTING OF LEGACIES FOR EACH CLASS BELOW

LegaciesMinstrel = 
{
	{
		name = "Weapon",
		defaultLegacy = {
			name = "Tactical Damage Rating",
			type = Type.TacticalDamageRating,
			modType = Modifier.Rating,
			mod = ModAdjustForLevelPerAge( ModifierTableTacticalDamageRating ),
			cost = CostTableDefault,
			maxRank = 7,
		},
		legacies = {
			{
				name = "Call of Orome Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {2,4.2, 6.5, 8.7, 11, 13.2, 15.5, 17.7, 20 },
				cost = CostTable36
			}, { -- RoI altered
				name = "Story of Courage Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {1,4,8,11,15,19,22,26,30},
				cost = CostTable18
			},{
				name = "Call of the Second Age Targets",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustModulo( {1,2,3,4,5}, 7 ),
				cost = CostAdjustModuloTarget( CostTableTarget1, CostTableAdjustTarget1 ),
				maxRank = MaxRankAdjustModulo( 5, 7 );
			},{
				name = "Call to Fate Critical Multiplier",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable36
			},{ -- RoI altered
				name = "Still as Death Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {5,19,33,48,62,76,91,105,120},
				cost = CostTable27
			},{
				name = "Soliloquy of Spirit Pulses",
				type = Type.Major,
				modType = Modifier.Number,
				mod = ModAdjustSoS( ModifierTableRating(1,1) ),
				cost = CostAdjustSoS( CostTable27 )
			}, { -- RoI altered
				name = "Rally! Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {10,46,82,118,155,191,227,263,300},
				cost = CostTable27
			}, { --RoI altered
				name = "Minor Ballad Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {2,5.5,9,12.5,16,19.5,23,26.5,30},
				cost = CostTable27
			}, { --RoI altered
				name = "Perfect Ballad Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {2,5.5,9,12.5,16,19.5,23,26.5,30},
				cost = CostTable27
			}, {
				name = "Piercing Cry Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {2,5.5,9,12.5,16,19.5,23,26.5,30},
				cost = CostTable27
			}, {
				name = "Target Resist Rating (Call to War)",
				type = Type.Major,
				modType = Modifier.Resist,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating460_50_neg ),
				cost = CostTable27
			}, {
				name = "Triumphant Spirit Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {5,19.4,33.8,48.1,62.5,76.9,91.3,105.6,120},
				cost = CostTable18
			}, {
				name = "Target Resist Rating (Invocation of Elbereth)",
				type = Type.Minor,
				modType = Modifier.Resist,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating460_50_neg ),
				cost = CostTable27
			}, { -- RoI altered
				name = "Anthem Duration",
				type = Type.Major,
				modType = Modifier.Duration,
				mod = {1,2.1,3.3,4.4,5.5,6.6,7.8,8.9,10},
				cost = CostTable36,
			}, {
				name = "Healing Threat",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {-1,-2,-3,-4,-6,-7,-8,-9,-10},
				cost = CostTable36
			}
		}
	},
	{
		name = "Song book",
		defaultLegacy = {
			name = "Tactical Healing Rating",
			type = Type.TacticalHealingRating,
			modType = Modifier.Rating,
			mod = ModAdjustForLevelAndAge( ModifierTableTacticalHealingRating ),
			cost = CostTableDefault,
			maxRank = 7,
		},
		legacies = 
		{
			{	--RoI altered
				name = "Anthem of War (Ranged Damage)",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,1.5,2,2.5,3,3.5,4,4.5,5},
				cost = CostTable27
			}, { -- TODO RoI (check)
				name = "Anthem of Composure (Resistance)",
				type = Type.Minor,
				modType = Modifier.Resist,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable27
			}, { -- TODO RoI (check)
				name = "Anthem of Composure (Tactical Defence)",
				type = Type.Minor,
				modType = Modifier.Mitigation,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable27
			}, { -- RoI altered
				name = "Anthem of War (Melee Damage)",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {2,3,4,5,6,7,8,9,10},
				cost = CostTable27
			}, { -- RoI altered
				name = "Anthem of War (Tactical Damage)",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,1.5,2,2.5,3,3.5,4,4.5,5},
				cost = CostTable27
			}, { -- RoI altered
				name = "Cry of the Chorus Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {5,19,33,48,62,76,91,105,120},
				cost = CostTable32
			}, { -- RoI altered
				name = "Chord of Salvation Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {1,2,3,4,5}, -- no adjust
				cost = { -- no adjust, CostTable63 truncated
					{27,41,59,81},
					{23,34,49,68},
					{18,27,39,54},
					{14,21,30,41}, -- check
					{9,14,20,27},
					{8,12,18,24}
				},
				maxRank = 5
			}, {
				name = "Healing & Motivation Skills Healing",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2,3,4,6,7,8,9,10},
				cost = CostTable32
			}, {
				name = "Bolster Courage Healing",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2,3,4,6,7,8,9,10},
				cost = CostTable36
			}, {
				name = "Echoes of Battle Resistance Debuff",
				type = Type.Minor,
				modType = Modifier.Resist,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19_neg ),
				cost = CostTable27
			}, {
				name = "Healing Skills Power Cost",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {-1,-2,-3,-4,-6,-7,-8,-9,-10 },
				cost = CostTable36
			}, {
				name = "Raise the Spirit Healing",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2,3,4,6,7,8,9,10},
				cost = CostTable36
			}, {
				name = "Song of the Dead Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {1,4,8,11,15,19,22,26,30 },
				cost = CostTable23
			}, {
				name = "Target Resist Rating (Songs)",
				type = Type.Minor,
				modType = Modifier.Resist,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19_neg ),
				cost = CostTable27
			}, {
				name = "Target Resist Rating (Ballads)",
				type = Type.Major,
				modType = Modifier.Resist,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating460_50_neg ),
				cost = CostTable32
			}, {
				name = "Inspire Fellows Healing",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2,3,4,6,7,8,9,10},
				cost = CostTable36
			}
		}
	}
}

LegaciesBurglar = 
{
	{
		name = "Weapon",
		defaultLegacy = {
			name = "DPS",
			type = Type.DPS,
			modType = Modifier.Rating,
			mod = ModAdjustForLevelPerAge( ModifierTableDPS1H ),
			cost = CostTableDefault,
			maxRank = 7,
		}, 
		legacies = 
		{
			{
				name = "Critical Response Skill Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable36
			}, {
				name = "Feint Attack Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {1,6,12,17,23,28,34,39,45},
				cost = CostTable27
			}, {
				name = "Cunning Attack Bleed Stack Chance",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {3,11,19,27,34,42,50,58,66},
				cost = CostTable32
			}, {
				name = "Burglar Skill Bleed Damage Multiplier",
				type = Type.Major, 
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable36
			}, {
				name = "Location is Everything Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {5,15,26,36,47,58,68,79,90},
				cost = CostTable18
			}, {
				name = "Lucky Strike Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {1,6,12,17,23,28,34,39,45},
				cost = CostTable27
			}, { -- TODO RoI 
				name = "Provoke Threat Up Rating",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = ModAdjustThreat( ModifierTableThreat, 18 ), -- TODO check actual table, total guess here
				cost = CostAdjustThreat( CostTable23, 18) -- TODO RoI, adjust ? total guess here
			}, { -- TODO RoI (check)
				name = "Riddle Range",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustRange( {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20} ),
				cost = CostAdjustRange( CostTable72 )
			}, {
				name = "Position Surprise Strike from Stealth",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {3,5.7,8.5,11.2,14,16.7,19.5,22.2,25},
				cost = CostTable36
			}, {
				name = "Ready and Able Cooldown",
				type = Type.Minor, 
				modType = Modifier.Cooldown,
				mod = {10,83,157,231,305,378,452,526,600},
				cost = CostTable36 -- TODO : CHECK (32 on L.I.P.S website, should be 36 though, need to check T6 to make sure)
			}, {
				name = "Small Snag Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {1,6,12,17,23,28,34,39,45},
				cost = CostTable27
			}, {
				name = "Subtle Stab Critical Multiplier",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {3,5.7,8.5,11.2,14,16.7,19.5,22.2,25},
				cost = CostTable36
			}, {
				name = "Surprise Strike Critical Rating",
				type = Type.Major,
				modType = Modifier.Number,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable36
			}, {
				name = "Subtle Stab Damage Multiplier",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable32
			}, {
				name = "Well Placed Strike Cooldown",
				type = Type.Minor, 
				modType = Modifier.Cooldown,
				mod = {4,11,18,25,32,39,46,53,60},
				cost = CostTable18
			}, { -- TODO RoI (check)
				name = "Trick Range",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustForLevelRangeAndAge( 
					{1,1.4, -- extrapolated
					1.8,2,2,2.6,3,3.4,3.8,4.2,4.7,5.1,5.5,5.9,6.3,7.1,7.5,7.9,8.3,8.7,9.1,9.5,9.9,10.3,10.7,11.1,11.5,
					11.9 -- extrapolated
					} 
				),
				cost = CostTable36
			}
		}
	},
	{
		name = "Tools",
		defaultLegacy = {
			name = "Incoming Healing Rating",
			type = Type.IncomingHealingRating,
			modType = Modifier.Rating,
			mod = ModAdjustForIncomingHealing( ModifierTableIncomingHealingRating ),
			cost = CostTableDefault,
			maxRank = 7,
		},
		legacies = 
		{
			{
				name = "Addle Induction Multiplier",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,7,13,19,26,32,38,44,50},
				cost = CostTable18
			}, {
				name = "Clever Retort Damage and Healing",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {4.5,8.5,13,17.5,22,26.5,31,35.5,40},
				cost = CostTable27
			}, {
				name = "Addle Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {1,2,3,4,5,6,7,8,10},
				cost = CostTable27
			}, {
				name = "Confound Countdown Duration",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {1,2,3,4,5,6,7,8,10},
				cost = CostTable32
			}, {
				name = "Aim Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {1,2,3,4,5,6,7,8,10},
				cost = CostTable18
			}, {
				name = "Gamble Chance",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {5,7,9,11,13,14,16,18,20},
				cost = CostTable27
			}, {
				name = "Hide in Plain Sight Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {7,29,50,72,93,115,136,158,180},
				cost = CostTable36
			}, {
				name = "Healing - Mischievous Glee",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {5,10.6,16.2,21.8,27.5,33.1,38.7,44.3,50},
				cost = CostTable36
			}, {
				name = "Off-hand Critical Chance",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ), 
				cost = CostTable32,
			}, {
				name = "Increased Range for Reveal Weakness",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = {1,3,5,8,10,12,15,17,20}, 
				cost = CostTable18
			}, {
				name = "Devastating Critical Buff Duration",
				type = Type.Major,
				modType = Modifier.Duration,
				mod = {3,6,9,13,16,19,23,26,30},
				cost = CostTable27
			}, { -- TODO RoI (check)
				name = "Stealth Level",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustStealth( {1,2,3} ),
				cost = CostAdjustStealth( CostTableStealth, CostTableAdjustStealth ),
				maxRank = MaxRankAdjustStealth(),
			}, {
				name = "Non-Enrage Trick Duration",
				type = Type.Major,
				modType = Modifier.Duration,
				mod = {1,2,4,6,8,9,11,13,15},
				cost = CostTable23
			}, {
				name = "Sneak Movement Speed",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2,3,4,6,7,8,9,10},
				cost = CostTable27
			}, { -- RoI altered
				name = "Pulse Modifier for Glee",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustModulo( {1,2,3,4,5,6}, 6),
				cost = CostAdjustModuloPulse( CostTablePulse2, CostTableAdjustPulse2 ),
				maxRank = MaxRankAdjustModulo( 6, 6 )
			}, {
				name = "Skills Critical Multiplier",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable36
			}, {
				name = "Positional Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable36
			}
		}
	}
}

LegaciesWarden = 
{
	{
		name = "Melee weapon",
		defaultLegacy = {
			name = "DPS",
			type = Type.DPS,
			modType = Modifier.DPS,
			mod = ModAdjustForLevelPerAge( ModifierTableDPS1H ),
			cost = CostTableDefault,
			maxRank = 7,
		}, 
		legacies = 
		{
			{
				name = "Ambush Critical Multiplier",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {3,5.7,8.5,11.2,14,16.7,19.5,22.2,25},
				cost = CostTable27
			}, {
				name = "Boar's Rush Critical Rating",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable27
			}, { -- TODO RoI (check)
				name = "War Cry Threat Up Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustThreat( ModifierTableThreat, 18 ),
				cost = CostAdjustThreat( CostTable23, 18 )
			}, {
				name = "Critical Strike Critical Multiplier",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {3,5.7,8.5,11.2,14,16.7,19.5,22.2,25},
				cost = CostTable27
			}, {
				name = "Wall of Steel Parry Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable27
			}, { -- TODO RoI (check)
				name = "Exultation of Battle Threat Up Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustThreat( ModifierTableThreat, 21 ),
				cost = CostAdjustThreat( CostTable36, 21 )
			}, {
				name = "Wages of Fear Positional Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.5,7.7,8.8,10},
				cost = CostTable27
			}, {
				name = "Hampering Javelin Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {1,2,4,6,8,9,11,13,15},
				cost = CostTable36
			}, {
				name = "Surety of Death Damage Over Time",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable27
			}, { -- TODO RoI (check)
				name = "Javelin Skills Max Range",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustRange( {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20} ),
				cost = CostAdjustRange( CostTable72 ),
			}, { -- TODO RoI (check)
				name = "Spear of Virtue Threat Up Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustThreat( ModifierTableThreat, 18 ),
				cost = CostAdjustThreat( CostTable23, 18 )
			}, {
				name = "Spear of Virtue Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable27
			}, {
				name = "Mighty Blow Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable27
			}, {
				name = "Onslaught Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable27
			}, { -- TODO RoI (check)
				name = "Piercing Strike Threat Up Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustThreat( ModifierTableThreat, 18 ),
				cost = CostAdjustThreat( CostTable23, 18 )
			}, {
				name = "Power, Power Attack Gambit Line",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {-1,-2,-3,-4,-6,-7,-8,-9,-10},
				cost = CostTable18
			}, {
				name = "Recklessnes Attack Duration",
				type  = Type.Major,
				modType = Modifier.Percent,
				mod = {-1,-2,-3,-4,-6,-7,-8,-9,-10},
				cost = CostTable23
			}, {
				name = "Power, Fierce Resolve Gambit Line",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {-1,-2,-3,-4,-6,-7,-8,-9,-10},
				cost = CostTable32
			}
		}
	}, {
		name = "Javelin",
		defaultLegacy = {
			name = "DPS",
			type = Type.DPS,
			modType = Modifier.DPS,
			mod = ModAdjustForLevelPerAge( ModifierTableDPS2H ),
			cost = CostTableDefault,
			maxRank = 7,
		},
		legacies = 
		{
			{
				name = "Dance of War Evade Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable27
			}, {
				name = "Ambush and Careful Step Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {-0.3,-0.6,-1,-1.3,-1.7,-2,-2.3,-2.7,-3},
				cost = CostTable36
			}, {
				name = "Celebration of Skill Healing",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,3,5,6,8,10,12,13,15},
				cost = CostTable23
			}, {
				name = "Conservation in Combat Power Regeneration",
				type = Type.Major,
				modType = Modifier.Number,
				mod = ModAdjustForLevelRangeAndAge( 
					{31,36.7,42.5,48.4,54.2,59.9,65.8,71.6,77.4,83.2,89,94.8,100.6,106.4,112.2,118,123.8,129.6,135.4,141.2,
					147,152.8,158.6,164.4,170.2,176,181.8,187.6 } -- TODO RoI - extrapolated
				),
				cost = CostTable36
			}, {
				name = "Adroit Maneuver Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {1,2,3,4,5,6,7,8,10},
				cost = CostTable32
			}, {
				name = "Careful Step Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {3,6,9,13,16,19,23,26,30},
				cost = CostTable36
			}, {
				name = "Resolution Resist Rating",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable27
			}, {
				name = "Determination In Combat Morale Regeneration",
				type = Type.Major,
				modType = Modifier.Number,
				mod = ModAdjustForLevelRangeAndAge(
					{65.3, 77.6, 89.8,102.1,114.3,126.5,138.8,151.1,163.3,175.6,187.8,200.1,212.3,224.6, -- 224.6 @ lvl 74 & 75 third age, TODO RoI check if changed for 2nd age
					236.8,249.1,261.4,273.6,285.8,298.1,310.3,
					322.6,334.8,347.1,359.3,371.6,383.8,396.1} -- TODO RoI, extrapolated
				),
				cost = CostTable36
			}, {
				name = "Javelin Skills To-Hit Movement Penalty",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {-2,-3,-4,-5,-6,-7,-8,-9,-10},
				cost = CostTable27
			}, {-- TODO RoI - make sure this does not adjust
				name = "Forced March Max Power Penalty",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2,3,4,5,6,7,8,9},
				cost = CostTable23
			}, { 
				name = "Dark Before Dawn Power Restored",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = ModAdjustSoS(
					{9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,26,26,27,28,29,30,31,32,33,34,35,36} -- TODO RoI check start value, 31+ extrapolated
				),
				cost = CostAdjustSoS( CostTable36 )
			}, {
				name = "Power, Safeguard Gambit Line",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {-1,-2,-3,-4,-6,-7,-8,-8,-10},
				cost = CostTable32
			}, {
				name = "Restoration Morale Healed",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,3,5,6,8,10,12,13,15},
				cost = CostTable23
			}, {
				name = "Shield Mastery Block Rating",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable27
			}, {
				name = "Safeguard Healing",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,3,5,6,8,10,12,13,15},
				cost = CostTable27
			}, {
				name = "Warden's Triumph Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {1,2.1,3.3,4.4,5.5,6.6,7.8,8.9,10},
				cost = CostTable36
			}, {
				name = "Heal Bonus for Conviction",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,3,5,6,8,10,12,13,15},
				cost = CostTable27
			}, {
				name = "Shield Piercer Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {1,2,4,6,8,9,11,13,15},
				cost = CostTable32
			}
		}
	}
}

LegaciesLoremaster = 
{
	{
		name = "Staff",
		defaultLegacy = {
			name = "Tactical Damage Rating",
			type = Type.TacticalDamageRating,
			modType = Modifier.Rating,
			mod = ModAdjustForLevelPerAge( ModifierTableTacticalDamageRating ),
			cost = CostTableDefault,
			maxRank = 7,
		}, 
		legacies = 	{
			{
				name = "Burning Ember Initial Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.8,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable36
			}, { -- TODO RoI (check)
				name = "Cracked Earth Range",
				type = Type.Major,
				modType = Modifier.Number,
				mod = ModAdjustRange( {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20} ),
				cost = CostAdjustRange( CostTable54 ),
			}, { -- TODO RoI,total guess atm regarding adjusts
				name = "Burning Embers Range",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustRange( {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20} ),
				cost = CostAdjustRange( CostTable54 ),
			}, {
				name = "Ents Go To War Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable32
			}, {
				name = "Cracked Earth Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable32
			}, {
				name = "Fire Skills Critical Multiplier",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {3,5.7,9.5,11.2,14,16.7,19.5,22.2,25},
				cost = CostTable32
			}, {
				name = "Lightning Skills Critical Rating",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable32
			}, {
				name = "Gust of Wind Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable32
			}, {
				name = "Light of the Rising Dawn Critical Multiplier",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable32
			}, {
				name = "Lightning Storm Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable36
			}, {
				name = "Light of the Rising Dawn Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable36
			}, {
				name = "Share the Power Increase",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2,3,4,6,7,8,9,10},
				cost = CostTable23
			}, {
				name = "Tactical Skills Direct Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable36
			}, {
				name = "Sticky Gourd Direct Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable32
			}, {
				name = "Staff Strike Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {1,1.5,2,2.5,3,3.5,4,4.5,5},
				cost = CostTable18
			}, {
				name = "Test of Will Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable36
			}, {
				name = "Storm Lore Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable32
			}
		}
	},
	{
		name = "Book",
		defaultLegacy = {
			name = "Tactical Healing Rating",
			type = Type.TacticalHealingRating,
			modType = Modifier.Rating,
			mod = ModAdjustForLevelAndAge( ModifierTableTacticalHealingRating ),
			cost = CostTableDefault,
			maxRank = 7,
		},
		legacies =
		{
			{ -- TODO RoI (check)
				name = "Ancient Craft Targets",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustModulo( {1,2,3,4,5}, 7 ),
				cost = CostAdjustModuloTarget( CostTableTarget1, CostTableAdjustTarget1 ),
				maxRank = MaxRankAdjustModulo( 5, 7 )
			}, { -- TODO RoI (check)
				name = "Bane Flare Target",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustModulo( {1,2,3,4,5}, 7 ),
				cost = CostAdjustModuloTarget( CostTableTarget1, CostTableAdjustTarget1 ),
				maxRank = MaxRankAdjustModulo( 5, 7 )				
			}, { -- TODO RoI (check)
				name = "Burning Embers Pulses",
				type = Type.Major,
				modType = Modifier.Number,
				mod = ModAdjustModulo( {1,2,3,4,5,6}, 6),
				cost = CostAdjustModuloPulse( CostTablePulse3, CostTableAdjustPulse3 ),
				maxRank = MaxRankAdjustModulo( 6, 6 )
			}, {
				name = "Cure Resistance Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {5,11,18,25,32,39,46,53,60},
				cost = CostTable18
			}, { -- TODO RoI (check)
				name = "Wizard Fire Pulses",
				type = Type.Major,
				modType = Modifier.Number,
				mod = ModAdjustModulo( {1,2,3,4,5,6}, 6),
				cost = CostAdjustModuloPulse( CostTablePulse3, CostTableAdjustPulse3 ),
				maxRank = MaxRankAdjustModulo( 6, 6 )
			}, {
				name = "Fire Damage Over Time",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable36
			}, {
				name = "Threat - Sign of the Wild Rage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,3,5,6,8,10,12,13,15},
				cost = CostTable23
			}, {
				name = "Test of Will Devastate Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable27
			}, {
				name = "Fire Skills Critical Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable32
			}, {
				name = "Target Resistance (Fire Skills)",
				type = Type.Major,
				modType = Modifier.Resist,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating460_50_neg ),
				cost = CostTable32
			}, { -- TODO RoI (check)
				name = "Fire-lore Targets",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustModulo( {1,2,3,4,5}, 7 ),
				cost = CostAdjustModuloTarget( CostTableTarget1, CostTableAdjustTarget1 ),
				maxRank = MaxRankAdjustModulo( 5, 7 )
			}, {
				name = "Healing from Beacon of Hope",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,3,5,6,8,10,12,13,15},
				cost = CostTable23
			}, {
				name = "Target Resistance (Debuffing Skills)",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating1105_105_neg ),
				cost = CostTable23
			}, {
				name = "Signs of Power Durations",
				type = Type.Major,
				modType = Modifier.Duration,
				mod = {3,6,9,13,16,19,23,25,30},
				cost = CostTable18
			}, {
				name = "Target Resistance (Damaging Skills)",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating460_50_neg ),
				cost = CostTable32
			}, {
				name = "Inner Flame Healing",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,7,13,19,26,32,38,44,50},
				cost = CostTable23
			}, { -- TODO RoI
				name = "Morale - Sign of the Wild Protection",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = { 1,2,3,4,5,6 },
				cost = { -- does not seem to adjust, TODO make sure
					{9,27,39,54,72},
					{8,22,32,44,59},
					{7,19,27,38,50},
					{6,16,22,31,41},
					{4,12,18,24,32},
					{3,9,13,17,23}
				},
				maxRank = 6,
			}
		}
	}
}

LegaciesChampion = 
{
	{
		name = "Weapon (One Handed)",
		defaultLegacy = {
			name = "DPS",
			type = Type.DPS,
			modType = Modifier.DPS,
			mod = ModAdjustForLevelPerAge( ModifierTableDPS1H ),
			cost = CostTableDefault,
			maxRank = 7,
		}, 
		legacies = 
		{
			{
				name = "Area of Effect Skill Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable36
			}, {
				name = "Combat-only Effect Removal Delay",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {1,1.5,2,2.5,3,3.5,4,4.5,5},
				cost = CostTable32
			}, {
				name = "Brutal Strikes Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable23
			}, {
				name = "Critical Damage Multiplier",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {5,11,16,22,28,33,39,44,50},
				cost = CostTable32
			}, {
				name = "Feral/Savage Strikes Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable23
			}, {
				name = "Damage for Relentless/Remorless Strikes",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable23
			}, {
				name = "Wild Attack Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable23
			}, {
				name = "Fear Nothing Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {5,15,26,36,47,58,68,79,90},
				cost = CostTable27
			}, {
				name = "Harmstring Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {1,2,3,4,6,7,8,9,10},
				cost = CostTable32
			}, { -- TODO: RoI (check)
				name = "Ferocious Strikes Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {1,2,3,4,5}, -- RoI values
				cost = { -- Rank 1 & 2 & 3 matches CostTable63
					{27,41,59,81}, -- cost on live
					{23,34,49,68}, -- costs on live
					{18,27,39,54}, -- check on live
					{14,21,30,41}, -- from CostTable63
					{9,14,20,27}, -- from CostTable63
					{8,12,18,24} -- from CostTable63
				},
				-- mod = {1,2,4,6,8,9,11,13,15}, -- Pre RoI
				-- cost = CostTable32 -- Pre RoI
				maxRank = 5
			}, {
				name = "Horn Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable23
			}, {
				name = "Increased Rend Armour Reduction",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge(
					{80,88, -- extrapolated TODO
					95,103,111,119,126,134,142,150,157,165,173,
					181,188,196,204,212, -- extrapolated TODO
					219,227,235,243,250,258,266,270,274,
					278 } -- extrapolated TODO
				),
				cost = CostTable27
			}, {
				name = "Battle Frenzy Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {1,3,5,8,10,12,15,17,20},
				cost = CostTable18
			}, {
				name = "Rend Bleed Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {3,5.7,8.5,11.2,14,16.7,19.5,22.2,25},
				cost = CostTable23
			}, { -- TODO  RoI changes (check)
				name = "Sudden Defence Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {5,19,34,48,63,77,91,106,120},
				cost = CostTable36,
			}, {
				name = "Sprint Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {3,6,9,13,16,19,23,26,30},
				cost = CostTable27
			}
		}
	},
	{
		name = "Weapon (Two Handed)",
		defaultLegacy = {
			name = "DPS",
			type = Type.DPS,
			modType = Modifier.DPS,
			mod = ModAdjustForLevelPerAge( ModifierTableDPS2H ),
			cost = CostTableDefault,
			maxRank = 7,
		},
		legacies = {}	-- legacies set to same as one handed below 
	},
	{
		name = "Rune",
		defaultLegacy = {
			name = "Incoming Healing Rating",
			type = Type.IncomingHealingRating,
			modType = Modifier.Rating,
			mod = ModAdjustForIncomingHealing( ModifierTableIncomingHealingRating ),
			cost = CostTableDefault,
			maxRank = 7,
		},
		legacies =
		{
			{
				name = "Blade Line Area Damage Power Cost",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {-1,-2,-3,-4,-6,-7,-8,-9,-10},
				cost = CostTable27
			}, { -- TODO RoI (check)
				name = "Ardour/Glory Pip Interval",
				type = Type.Major,
				modType = Modifier.Number,
				mod = ModAdjustModulo( {-1,-2,-3,-4}, 8 ),
				cost = CostAdjustModuloPipInterval( CostTablePipInterval, CostTableAdjustPipInterval ),
				maxRank = MaxRankAdjustModulo( 4, 8 )
			}, {
				name = "Ardour/Glory in-Combat Power Regen",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,3,5,6,8,10,12,13,15},
				cost = CostTable23
			}, { -- RoI altered
				name = "Adamant/Invincible Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {1,2,3,4,5},
				cost = CostTable36,
				maxRank = 5;
			}, {
				name = "Ardour/Glory Parry and Evade Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable27
			}, { -- TODO RoI (check)
				name = "Harmstring Range",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustHarmstringRange( {0.5,1,2} ),
				cost = CostAdjustHarmstringRange( CostTableHarmstringRange, CostTableAdjustHarmstringRange ),
				maxRank = MaxRankAdjustHarmstringRange()
			}, {
				name = "Heroics Morale Heal Chance",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,11,21,31,41,50,60,70,80},
				cost = CostTable18
			}, {
				name = "Bracing Attack Heal",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,3,5,6,8,10,12,13,15},
				cost = CostTable27
			}, {
				name = "Dire Need Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {4,11,18,25,32,39,46,53,60}, -- TODO: RoI (checked)
				cost = CostTable32
			}, {
				name = "Ebbing Ire Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {4,11,18,25,32,39,46,53,60},
				cost = CostTable36
			}, { -- TODO: RoI changes (checked)
				name = "(Glory) Hedge Armour Rating Mod",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustForLevelRangeAndAge( 
					{ 50,85,120,155,190,225,260,295,330,365,400,435,470, -- TODO Extrapolated
					505,540,575,610,645,680,715,750,
					785, -- TODO Check
					820,855,890,925,960,995 }  -- TODO Extrapolated
				),
				cost = CostTable27
			}, {
				name = "Strikes Line Power Cost",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {-1,-2,-3,-4,-6,-7,-8,-9,-10},
				cost = CostTable27
			}, {
				name = "Swift Strike/Swift Blade Power Cost",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {-2,-6,-9,-13,-16,-20,-23,-27,-30},
				cost = CostTable27
			}, {
				name = "Stun Duration - Champion's Horn",
				type = Type.Major,
				modType = Modifier.Duration,
				mod = {1,1.5,2,2.5,3,3.5,4,4.5,5},
				cost = CostTable36
			}, {
				name = "Fight On Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {3,6,9,13,16,19,23,26,30},
				cost = CostTable32
			}, { -- TODO RoI (check)
				name = "Rend Pulses",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustModulo( {1,2,3,4,5,6}, 6 ),
				cost = CostAdjustModuloPulse( CostTablePulse1, CostTableAdjustPulse1 ),
				maxRank = MaxRankAdjustModulo( 6, 6 ),
			}, { -- TODO: RoI (check)
				name = "Battle Acuity Duration",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = {1,2,3,4,5,6,7,8,10},
				cost = CostTable36
			}
		}
	}
}
LegaciesChampion[2].legacies = LegaciesChampion[1].legacies;

LegaciesCaptain = 
{
	{
		name = "Weapon (two handed)",
		defaultLegacy = {
			name = "DPS",
			type = Type.DPS,
			modType = Modifier.DPS,
			mod = ModAdjustForLevelPerAge( ModifierTableDPS2H ),
			cost = CostTableDefault,
			maxRank = 7,
		}, 
		legacies = 
		{
			{
				name = "Blade of Elendil Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {3,5.7,8.5,11.2,14,16.7,19.5,22.2,25},
				cost = CostTable32
			}, {
				name = "Cry Power Cost",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {-1,-2,-3,-4,-6,-7,-8,-9,-10},
				cost = CostTable18b
			}, {
				name = "Defensive Strike Armour Buff",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating( 50, 35 ) ), -- TODO RoI
				cost = CostTable23
			}, {
				name = "Devastating Blow Critical Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable23
			}, {
				name = "Cutting Attack Bleed Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {4,8.5,13,17.5,22,26.5,31,35.5,40},
				cost = CostTable27
			}, {
				name = "Increased Duration of Make Haste",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {1,4,7,10,13,16,19,22,25},
				cost = CostTable18
			}, {
				name = "Kick Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {1,4,8,11,15,19,22,26,30},
				cost = CostTable27
			}, { -- TODO RoI (check)
				name = "Pressing Attack Max Targets",
				type = Type.Major,
				modType = Modifier.Number,
				mod = ModAdjustModulo( {1,2,3,4,5}, 7 ),
				cost = CostAdjustModuloTarget( CostTableTarget2, CostTableAdjustTarget2 ),
				maxRank = MaxRankAdjustModulo( 5, 7 ),
			}, {
				name = "Melee Skill Power Cost",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {-1,-2,-3,-4,-6,-7,-8,-9,-10},
				cost = CostTable23
			}, {
				name = "Pressing Attack Critical Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable23
			}, {
				name = "Routing Cry Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {4,8.5,13,17.5,22,26.5,31,35.5,40},
				cost = CostTable27
			}, {
				name = "Tactics On Guard Parrry Rating Buff",
				type = Type.Minor, 
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable36
			}, {
				name = "Tactics: Relentless Attack Critical Rating Buff",
				type = Type.Minor, 
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable27
			}, {
				name = "Telling Mark Damage",
				type = Type.Major, 
				modType = Modifier.Percent,
				mod = {2,2.3,2.7,3.1,3.5,3.8,4.2,4.6,5},
				cost = CostTable36
			}, {
				name = "Morale from Motivating Speech",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,1.5,2,2.5,3,3.5,4,4.5,5},
				cost = CostTable36
			}, {
				name = "To Arms Duration",
				type = Type.Major, 
				modType = Modifier.Duration,
				mod = {1,2,4,6,8,9,11,13,15},
				cost = CostTable27
			}
		}
	},
	{
		name = "Weapon (One Handed)",
		defaultLegacy = {
			name = "DPS",
			type = Type.DPS,
			modType = Modifier.DPS,
			mod = ModAdjustForLevelPerAge( ModifierTableDPS1H ),
			cost = CostTableDefault,
			maxRank = 7,
		},
		legacies = {}	-- legacies set to same as two handed below 
	},	
	{
		name = "Emblem",
		defaultLegacy = {
			name = "Tactical Healing Rating",
			type = Type.TacticalHealingRating,
			modType = Modifier.Rating,
			mod = ModAdjustForLevelAndAge( ModifierTableTacticalHealingRating ),
			cost = CostTableDefault,
			maxRank = 7,
		},
		legacies =
		{
			{
				name = "Captain's Escape from Darkness Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {10,83,157,231,305,378,452,526,600},
				cost = CostTable36
			}, { -- TODO : RoI (checked)
				name = "Shield of the Dunedain Cooldown",
				type = Type.Minor, 
				modType = Modifier.Cooldown,
				mod = {5,10,15,20,30,45,60,75,90}, -- TODO RoI (checked)
				cost = CostTable36
			}, { -- TODO: RoI (check)
				name = "Words of Courage Pulses",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustPulse(),
				cost = CostAdjustPulse( CostTable54 ),
				maxRank = MaxRankAdjustPulse()

--[[				{ -- CostTable54 ?
					{0,0,0,0,0,0,0,0},
					{9,27,39,54,72,93,117,144},
					{0,0,0,0,0,0,0,0},
					{0,14,20,27,36,47,59,72}, -- 5 @ lvl 65 third age
					{0,0,0,0,0,0,0,0},
					{0,0,0,0,0,0,0,0},
				},--]]
				--[[ name = "Duration of In Defence of Middle-Earth",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {30,50,70,90,110,130,150,170,190},
				cost = CostTable18--]]
			}, {
				name = "Grave Wound Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {1,1.5,2,2.5,3,3.5,4,4.5,5},
				cost = CostTable18
			}, {
				name = "Healing from Rallying Cry",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,3,5,6,8,10,12,13,15},
				cost = CostTable23
			}, {
				name = "Healing from Strength of Will",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,3,5,6,8,10,12,13,15},
				cost = CostTable27
			}, {
				name = "Rallying Cry Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {1,4,8,11,15,19,22,26,30},
				cost = CostTable32
			}, {
				name = "Melee Skills Healing",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,3,5,6,8,10,12,13,15},
				cost = CostTable27
			}, {
				name = "Muster Courage Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {1,2,3,4,5,6,7,8,10},
				cost = CostTable27b
			}, {
				name = "Shadow's Lament Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {1,2,4,6,8,9,11,13,15},
				cost = CostTable32
			}, {
				name = "Time of Need Morale Cost",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,7,13,19,26,32,38,44,50},
				cost = CostTable23
			}, {
				name = "Valiant Strike Morale Return",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2,3,4,6,7,8,9,10},
				cost = CostTable27
			}, { -- TODO RoI (check)
				name = "Tactics: Focus in-Combat Power Regen Buff",
				type = Type.Major,
				modType = Modifier.Number,
				mod = ModAdjustForLevelRangeAndAge( 
					{30.6,36.6,42,48,54,59.4,65.4,71.4,77.4,82.8,88.8,94.8,100.2,106.2,112.2,117.6,123.6,129.6,135,141,147,
					153,159,165,171,176,181,187 }-- TODO RoI,: extrapolated
				),
				cost = CostTable32
			}, { -- TODO RoI (check)
				name = "Time of Need Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {15,30,45,60,75,90,110,130,150}, -- TODO RoI (checked)
				cost = CostTable23
			}, {
				name = "Vocal Skills Healing",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,3,5,6,8,10,12,13,15},
				cost = CostTable27
			}, {
				name = "Words of Courage Power and Morale Cost",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {-1,-2,-3,-4,-6,-7,-8,-9,-10},
				cost = CostTable36
			}
		}
	}
}

LegaciesCaptain[2].legacies = LegaciesCaptain[1].legacies;

LegaciesGuardian = 
{
	{
		name = "Weapon",
		defaultLegacy = {
			name = "DPS",
			type = Type.DPS,
			modType = Modifier.DPS,
			mod = ModAdjustForLevelPerAge( ModifierTableDPS1H ),
			cost = CostTableDefault,
			maxRank = 7,
		}, 
		legacies = 
		{
			{
				name = "Charge Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {1,2,3,4,6,7,8,9,10},
				cost = CostTable18
			}, { -- TODO RoI (check)
				name = "Guardian Area Effect Melee Targets",
				type = Type.Major,
				modType = Modifier.Number,
				mod = ModAdjustModulo( {1,2,3,4,5}, 7 ),
				cost = CostAdjustModuloTarget( CostTableTarget1, CostTableAdjustTarget1 ),
				maxRank = MaxRankAdjustModulo( 5, 7 ),
			}, { -- TODO : 1h only ?
				name = "Bash Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable18
			}, { -- TODO: 2h only ?
				name = "Brutal Assault Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable27
			}, {
				name = "Guardian Bleed Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {2,4.2,6.5,8.7,11,13.2,15.5,17.7,20},
				cost = CostTable36
			}, { -- TODO RoI (check)
				name = "Guardian Bleed Pulses",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustModulo( {1,2,3,4,5,6}, 6 ),
				cost = CostAdjustModuloPulse( CostTablePulse1, CostTableAdjustPulse1 ),
				maxRank = MaxRankAdjustModulo( 6, 6 ),
			}, {
				name = "Overwhelm Critical Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( 
					{884,968,1052,1136,1220,1304,1388,1472,1556,1640,1724,1808,1892,1976,2060,2144,2228,2312,2396,2428,2464,2499,
					2534,2569,2604,2640,2675, 
					2710} -- extrapolated
				),
				cost = CostTable32
			}, {
				name = "Stamp Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {1,4,8,11,15,19,22,26,30},
				cost = CostTable27
			}, {
				name = "Sweeping Cut Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable27
			}, {
				name = "Guardian's Ward Duration",
				type = Type.Major,
				modType = Modifier.Duration,
				mod = {1,2,3,4,5,6,7,8,10},
				cost = CostTable32
			}, { -- TODO RoI (check)
				name = "Targeted Melee Skill Range",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustForLevelRangeAndAgeCap( 
					{	
						0.5,0.6, -- extrapolated
						0.8,0.9,1.1,1.2,1.4,1.5,1.7,1.8,2.0,2.1,2.3,2.4,2.6,2.7,2.9
					},
					8
				),
				cost = CostTable36,
				maxRank = MaxRankAdjustCap( 8 )
			}, {
				name = "Guardian's Pledge Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {4,11,18,25,32,39,46,53,60},
				cost = CostTable32
			}, {
				name = "Whirling Retaliation Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable27
			}, { -- TODO 1H only ?
				name = "Vexing Blow Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable18
			}, {
				name = "Turn the Tables Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {5,15,26,36,47,58,68,79,90},
				cost = CostTable32
			}, {
				name = "Warrior's Heart Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {5,11,18,25,32,39,46,53,60},
				cost = CostTable27
			}, {
				name = "To the King Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable27
			}, { -- TODO RoI (check)
				name = "Overpower Power Cost",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {-1,-2,-3,-4,-5,-6},
				cost = { -- TODO: Varying costs ?
					{16,48,69,95,126}, -- check
					{14,41,59,81,108},
					{12,34,49,68,90},
					{9,27,39,54,72},
					{7,21,30,41,54},
					{5,14,20,27,36}
				},
				maxRank = 6
			}
		}
	},
	{
		name = "Weapon (Two Handed)",
		defaultLegacy = {
			name = "DPS",
			type = Type.DPS,
			modType = Modifier.DPS,
			mod = ModAdjustForLevelPerAge( ModifierTableDPS2H ),
			cost = CostTableDefault,
			maxRank = 7,
		},
		legacies = {}	-- legacies set to same as one handed below 
	},	
	{
		name = "Belt",
		defaultLegacy = {
			name = "Shield Use Rank",
			type = Type.ShieldUse,
			modType = Modifier.Rating,
			mod = ModAdjustForLevelPerAge( ModifierTableShieldUseRating ),
			cost = CostTableDefault,
			maxRank = 7,
		},
		legacies =
		{
			{
				name = "Catch a Breath Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {1,4,8,11,15,19,22,26,30},
				cost = CostTable27
			}, {
				name = "Catch a Breath Morale Heal",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = { 1,3,5,6,8,10,12,13,15},
				cost = CostTable27
			}, {
				name = "Cry Resist Chance",
				type = Type.Major,
				modType = Modifier.Rating, 
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable32
			}, { -- TODO RoI (check)
				name = "Challenge Targets",
				type = Type.Major,
				modType = Modifier.Number,
				mod = ModAdjustModulo( {1,2,3,4,5}, 7 ),
				cost = CostAdjustModuloTarget( CostTableTarget3, CostTableAdjustTarget3 ),
				maxRank = MaxRankAdjustModulo( 5, 7 ),
			}, {
				name = "Guardian's Threat Block and Parry Rating",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable32
			}, {
				name = "Guardian's Ward Block Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable32
			}, {
				name = "Guardian's Ward Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable18
			}, {
				name = "Power Restore on Reactions",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {50,88,125,163,200,238,275,313,350},
				cost = CostTable27
			}, {
				name = "Guardian's Ward Parry Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable32
			}, {
				name = "Reactive Block Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {300,625,750,975,1200,1425,1650,1875,2100}, -- TODO RoI
				cost = CostTable18
			}, { -- TODO RoI
				name = "Increased Block, Parry and Evade from Protection",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = {122,132,141,151,160,170,179,189,198}, --TODO check adjust ? correct value for lvl 61 third age
				cost = CostTable32
			}, {
				name = "Shield Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {2,4,7,10,13,16,19,22,25},
				cost = CostTable63
			}, { -- TODO RoI (check)
				name = "Shield Wall Range",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustRange( {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20} ),
				cost = CostAdjustRange( CostTable36 )
			}, {
				name = "Shield Smash Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {2,4,7,10,13,16,19,22,25},
				cost = CostTable32
			}, {
				name = "Stagger Critical Rating",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating1105_105 ),
				cost = CostTable32
			}, { -- TODO RoI (check)
				name = "Threat Generation Up Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustThreat( ModifierTableThreat, 18 ),
				cost = CostAdjustThreat( CostTable36, 18 )
			}, {
				name = "Stagger Effect Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {1,2,3,4,5,6,7,8,10},
				cost = CostTable27
			}, {
				name = "Warrior's Heart/Fortitude Morale Heal",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,4,7,10,13,15,19,22,25},
				cost = CostTable23
			}, {
				name = "Stagger Positional Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.7,4.6,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable27
			}, { 
				name = "Sting Damage", 
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = {Cost54, Cost45, Cost36, Cost27, Cost23, Cost18 }
			}
		}
	}
}
LegaciesGuardian[2].legacies = LegaciesGuardian[1].legacies;

LegaciesHunter = 
{
	{
		name = "Ranged weapon",
		defaultLegacy = {
			name = "DPS",
			type = Type.DPS,
			modType = Modifier.DPS,
			mod = ModAdjustForLevelPerAge( ModifierTableDPS2H ),
			cost = CostTableDefault,
			maxRank = 7,
		}, 
		legacies = 
		{
			{
				name = "Burn Hot Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.1,3.2,4.3,5.5,6.6,7.7,8.8,10},
				cost = CostTable27
			}, {
				name = "Distracting Shot Resistance Penetration",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating460_50 ),
				cost = CostTable36
			}, { -- TODO RoI (check)
				name = "Endurance Quick Shot Threat Down Rating",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = {1,2,3,4,5,6,7,8,9}, -- does not seem to currently adjust
				cost = CostTable27
			}, {
				name = "Distracting Shot Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {1,2,4,6,8,9,11,13,15},
				cost = CostTable36
			}, {
				name = "Barbed Arrow Bleed Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {6,11.5,17,22.5,28,33.5,39,44.5,50},
				cost = CostTable32
			}, { -- TODO RoI (check)
				name = "Focus Bow Threat Down Rating",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = ModAdjustThreat( ModifierTableThreat_neg, 19 ), 
				cost = CostAdjustThreat( CostTable32, 19 )
			}, {
				name = "Induction Bow Power Cost",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {-1,-2,-3,-4,-6,-7,-8,-9,-10},
				cost = CostTable27
			}, {
				name = "Heart Seeker Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {2,4.2,6.5,8.7,11,13.2,15.5,17.7,20},
				cost = CostTable36
			}, {
				name = "Induction Bow Critical Multiplier",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {3,5.7,8.5,11.2,14,16.7,19.5,22.2,25},
				cost = CostTable36
			}, {
				name = "Focus Bow Critical Multiplier",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {3,5.7,8.5,11.2,14,16.7,19.5,22.2,25},
				cost = CostTable36
			}, {
				name = "Focus Bow Power Cost",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {-1,-2,-3,-4,-6,-7,-8,-9,-10},
				cost = CostTable27
			}, { -- TODO RoI (check)
				name = "Quick Shot Critical Chance",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,1.5,2,2.5,3,3.5,4,4.5,5},
				cost = CostTable27
				--[[ pre RoI 
				name = "Quick Shot Critical Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = {517,565,612,660,707,755,802,850,897,945,992,1040,1087,1135}, -- TODO Double check last three values
				adjust = Adjust.LevelRangeAge,
				cost = CostTable27--]]
			}, {
				name = "Ranged Skill Evade Chance Modifier",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19_neg ), 
				cost = CostTable32
			}, {
				name = "Ranged Evade Rating",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ), 
				cost = CostTable32
			}, { -- TODO RoI (check)
				name = "Induction Bow Threat Down Rating",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = ModAdjustThreat( ModifierTableThreat_neg, 19 ),
				cost = CostAdjustThreat( CostTable32, 19 ),
			}, {
				name = "Ranged Skill Block Chance Modifier",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable32
			}, {
				name = "Merciful Shot Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {1,2,3,4,5,6,7,8,10},
				cost = CostTable32
			}, {
				name = "Strength Quick Shot Slow",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2,3,4,6,7,8,9,10},
				cost = CostTable32
			}
		}
	},
	{
		name = "One-handed Weapon",
		defaultLegacy = {
			name = "DPS",
			type = Type.DPS,
			modType = Modifier.DPS,
			mod = ModAdjustForLevelPerAge( ModifierTableDPS1H ),
			cost = CostTableDefault,
			maxRank = 7,
		},
		legacies =
		{
			{
				name = "Agile Rejoinder Heal Chance",
				type = Type.Minor;
				modType = Modifier.Percent,
				mod = {2,14,27,39,51,63,76,88,100},
				cost = CostTable23
			}, { -- TODO: RoI (check)
				name = "Blindside Critical Rating",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge(
					{517,565,612,660,707,755,802,850,897,945,992,1040,1087,1135,1182,1230,1277,1325,1340,1360,1380,1400,
					1420,1440,1460,1480,1500,1520} -- TODO:  last values extrapolated
				),
				cost = CostTable23
			}, {
				name = "Desperate Flight Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {10,83,157,231,305,378,452,526,600},
				cost = CostTable32
			}, {
				name = "Critical Magnitude in Precision Stance",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,3,4,5,7,8,10,11,13},
				cost = CostTable32
			}, {
				name = "Beneath Notice Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {5,19,33,48,62,76,91,105,120},
				cost = CostTable18
			}, {
				name = "Find the Path Movement Speed",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2,3,4,5,6},
				cost = { -- = cost table 27, truncated, rank 2 costs / 2
					{12,34,49,68,90}, -- 12,34,49,68,90 on lvl 75 3rd age
					{9,27,39,54,72},
					{7,21,30,41,54}, -- 7 @ lvl 53 third age, 7 @ lvl 60 3rd age, 7@ lvl 59 3rd age
					{5,14,20,27,36}, -- 5 @ lvl 65 third age, 5 @ lvl 70 3rd age
					{4,12,18,24,32}, -- checked rank3+ costs. R2:  4 on level 65 2nd age 
					{4,11,15,21,27}-- first age lvl 65 : 4, 11, 15, 21, 27
				}, -- costs don't asjust for this one
				maxRank = 6
			}, {
				name = "Improved Strength of Earth Morale Regen",
				type = Type.Major,
				modType = Modifier.Rating,
				mod = ModAdjustSoS( ModifierTableRating( 60, 60 ) ), -- RoI Check (checked to lvl 74 3rd age)
				cost = CostAdjustSoS( CostTable18 )
			}, { -- TODO RoI (check)
				name = "Maximum Targets for AoE Skills",
				type = Type.Major,
				modType = Modifier.Number,
				mod = ModAdjustModulo( {1,2,3,4,5}, 7),
				cost = CostAdjustModuloTarget( CostTableTarget3, CostTableAdjustTarget3 ),
				maxRank = MaxRankAdjustModulo( 5, 7 )
			}, {
				name = "Low Cut Bleed Chance",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,7,13,19,26,32,38,44,50},
				cost = CostTable23 -- checked T6 lvl 65 3rd age
			}, {
				name = "Melee Critical Multiplier",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {3,5.7,8.5,11.2,14,16.7,19.5,22.2,25},
				cost = CostTable36
			}, {
				name = "Melee Critical Rating",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable36
			}, {
				name = "Needful Haste Duration",
				type = Type.Major,
				modType = Modifier.Duration,
				mod = {1,2,4,6,8,9,11,13,15},
				cost = CostTable32
			}, { -- TODO RoI (check)
				name = "Stealth Detection",
				type = Type.Minor,
				modType = Modifier.Number,
				mod = ModAdjustStealth( {1,2,3} ),
				cost = CostAdjustStealth( CostTableStealth, CostTableAdjustStealth ),
				maxRank = MaxRankAdjustStealth(),
			}, {
				name = "Power Restored by Deep Concentration",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {2,6,9,13,16,20,23,27,30},
				cost = CostTable27
			}, {
				name = "Swift Stroke Parry and Evade Rating",
				type = Type.Minor,
				modType = Modifier.Rating,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating207_19 ),
				cost = CostTable18
			}, {
				name = "Press Onward Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {5,15,26,36,47,58,68,79,90},
				cost = CostTable32
			}
		}
	}
}

LegaciesRunekeeper = 
{
	{
		name = "Rune-stone",
		defaultLegacy = {
			name = "Tactical Damage Rating",
			type = Type.TacticalDamageRating,
			modType = Modifier.Rating,
			mod = ModAdjustForLevelPerAge( ModifierTableTacticalDamageRating ),
			cost = CostTableDefault,
			maxRank = 7,
		}, 
		legacies = 
		{
			{	
				name = "Fire Skill Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable36
			}, {
				name = "Battle Attuned Skill Power Cost",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {-1,-3,-5,-6,-8,-10,-12,-13,-15},
				cost = CostTable27
			}, {
				name = "Chill of Winter Debuff Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {1,2,3,4,5,6,7,8,10},
				cost = CostTable36
			}, {
				name = "Chill of Winter Damage",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable36
			}, {
				name = "Distracting Flame Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {1,4,8,11,15,19,22,26,30},
				cost = CostTable27
			}, {
				name = "Fury of Storm Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable36
			}, {
				name = "Healing",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,1.5,2,2.5,3,3.5,4,5,6},
				cost = CostTable36
			}, {
				name = "Glorious Foreshadowing Duration",
				type = Type.Minor,
				modType = Modifier.Duration,
				mod = {3,6,9,13,16,19,23,26,30},
				cost = CostTable32
			}, {
				name = "Healing Over Time",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2,3,4,6,7,8,9,10},
				cost = CostTable36
			}, {
				name = "Healing Attuned Skill Power Cost",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {-1,-3,-5,-6,-8,-10,-12,-13,-15},
				cost = CostTable27
			}, {
				name = "Fury of Storm Critical Multiplier",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {2,5.5,9,12.5,16,19.5,23,26.5,30},
				cost = CostTable36
			}, { -- TODO RoI (check)
				name = "Vivid Imagery Targets",
				type = Type.Major,
				modType = Modifier.Number,
				mod = ModAdjustModulo( {1,2,3,4,5}, 7),
				cost = CostAdjustModuloTarget( CostTableTarget3, CostTableAdjustTarget3 ),
				maxRank = MaxRankAdjustModulo( 5, 7 ),
			}, {
				name = "Wrath of Flame Damage Over Time",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.7,4.5,6.2,8,9.7,11.5,13.2,15},
				cost = CostTable36
			}, {
				name = "Steady Attuned Skill Power",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {-1,-3,-5,-6,-8,-10,-12,-13,-15},
				cost = CostTable23
			}, { -- TODO RoI (check)
				name = "Finesse Rating", 
				type = Type.Minor,
				modType = Modifier.Resist,
				mod = ModAdjustForLevelRangeAndAge( 
					{ 
						258,282,306,330, -- extrapolated
						353, 377, 401, 425, 448, 472, 496, 520, 543,
						567,591,615,638,662,670, -- extrapolated:
						680,690,700,710,720,730,740,750,
						760 -- extrapolated
					}
				),
				cost = CostTable36
			}
		}
	},
	{
		name = "Rune-satchel",
		defaultLegacy = {
			name = "Tactical Healing Rating",
			type = Type.TacticalHealingRating,
			modType = Modifier.Rating,
			mod = ModAdjustForLevelAndAge( ModifierTableTacticalHealingRating ),
			cost = CostTableDefault,
			maxRank = 7,
		},
		legacies =
		{
			{
				name = "Chilling Rhetoric Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {1,2,3,4,5,6,7,8,10},
				cost = CostTable18
			}, {
				name = "Ceaseless Argument Power Cost",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {-1,-3,-5,-6,-8,-10,-12,-13,-15},
				cost = CostTable27
			}, {
				name = "Distracting Winds Cooldown",
				type = Type.Major,
				modType = Modifier.Cooldown,
				mod = {4,11,18,25,32,39,46,53,60},
				cost = CostTable32
			}, {
				name = "Do Not Fall This Day Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {5,23,43,63,83,101,121,140,160},
				cost = CostTable36
			}, {
				name = "Epic for the Ages Healing",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2,3,4,6,7,8,9,10},
				cost = CostTable36
			}, {
				name = "Essence of Flame Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {2,4.2,6.5,8.7,11,13.2,15.5,17.7,20},
				cost = CostTable36
			}, {
				name = "Mending Verse Healing",
				type = Type.Minor,
				modType = Modifier.Percent,
				mod = {1,2,3,4,6,7,8,9,10},
				cost = CostTable36
			}, {
				name = "Frozen Epilogue Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {5,24,43,63,82,101,121,140,160},
				cost = CostTable32
			}, {
				name = "Pulses Prelude to Hope",
				type = Type.Major,
				modType = Modifier.Number,
				mod = ModAdjustPulse(),
				cost = CostAdjustPulse( CostTable36 ),
				maxRank = MaxRankAdjustPulse()
			}, {
				name = "Scribe's Spark Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {2,4.2,6.5,8.7,11,13.2,15.5,17.7,20},
				cost = CostTable27
			}, {
				name = "Self Motivation Power Return",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2.8,4.5,6.3,8,9.8,11.5,13.3,15},
				cost = CostTable32
			}, {
				name = "Shocking Words Stun Chance",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,3,6,8,11,13,15,18,20},
				cost = CostTable18
			}, {
				name = "Writ of Fire Damage",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {2,4.2,6.5,8.7,11,13.2,15.5,17.7,20},
				cost = CostTable32
			}, {
				name = "Writ of Health Healing",
				type = Type.Major,
				modType = Modifier.Percent,
				mod = {1,2,3,4,6,7,8,9,10},
				cost = CostTable32
			}, {
				name = "Word of Exaltation Cooldown",
				type = Type.Minor,
				modType = Modifier.Cooldown,
				mod = {1,4,8,11,15,19,22,26,30},
				cost = CostTable27
			}, {
				name = "Target Shocking Touch Resist Rating",
				type = Type.Minor,
				modType = Modifier.Resist,
				mod = ModAdjustForLevelRangeAndAge( ModifierTableRating( -690, -75 ) ), -- TODO RoI
				cost = CostTable27
			}
		}
	}
}

-- Stat legacies 
ModifierTableStat = {5,7,8,10,11,13,14,16,17,19,20,22,24,26,28,30,31,32,33,34,35,36,37,38,39,40,41,42}; -- TODO check 42
LegaciesStat =
{
	{
		name = "Agility",
		type = Type.Stat,
		modType = Modifier.Number,
		mod = ModAdjustForLevelRangeAndAge( ModifierTableStat ),
		cost = CostTable23
	}, {
		name = "Fate",
		type = Type.Stat,
		modType = Modifier.Number,
		mod = ModAdjustForLevelRangeAndAge( ModifierTableStat ),
		cost = CostTable23
	}, {
		name = "Might",
		type = Type.Stat,
		modType = Modifier.Number,
		mod = ModAdjustForLevelRangeAndAge( ModifierTableStat ),
		cost = CostTable23		
	}, {
		name = "Vitality",
		type = Type.Stat,
		modType = Modifier.Number,
		mod = ModAdjustForLevelRangeAndAge( ModifierTableStat ),
		cost = CostTable23
	}, {
		name = "Will",
		type = Type.Stat,
		modType = Modifier.Number,
		mod = ModAdjustForLevelRangeAndAge( ModifierTableStat ),
		cost = CostTable23
	}
}
